(function e(t,n,r){function s(o,u){if(!n[o]){if(!t[o]){var a=typeof require=="function"&&require;if(!u&&a)return a(o,!0);if(i)return i(o,!0);throw new Error("Cannot find module '"+o+"'")}var f=n[o]={exports:{}};t[o][0].call(f.exports,function(e){var n=t[o][1][e];return s(n?n:e)},f,f.exports,e,t,n,r)}return n[o].exports}var i=typeof require=="function"&&require;for(var o=0;o<r.length;o++)s(r[o]);return s})({1:[function(require,module,exports){
/// <reference path="../ts_definitions/ScreenCalcTypes.d.ts" />
var ScreenMath = require('./ScreenMath');

/**
* Allows screen information to be easily stored and calculated
*/
var ScreenCalc = (function () {
    function ScreenCalc(properties) {
        this.pixelWidth = null;
        this.pixelHeight = null;
        this.pixelCount = null;
        this.pixelDensity = null;
        this.ratio = null;
        this.physicalWidth = null;
        this.physicalHeight = null;
        this.diagonalSize = null;
        if (typeof properties !== "undefined") {
            this.setData(properties);
        }
    }
    /**
    * Set data for one or more properties.
    * If 'replace' param is set to true, properties that aren't passed will be set to null.
    */
    ScreenCalc.prototype.setData = function (properties, replace) {
        if (typeof replace === "undefined") { replace = false; }
        if (replace) {
            this.pixelWidth = null;
            this.pixelHeight = null;
            this.pixelCount = null;
            this.pixelDensity = null;
            this.ratio = null;
            this.physicalWidth = null;
            this.physicalHeight = null;
            this.diagonalSize = null;
        }

        if (typeof properties.pixelWidth !== "undefined") {
            this.setPixelWidth(properties.pixelWidth);
        }

        if (typeof properties.pixelHeight !== "undefined") {
            this.setPixelHeight(properties.pixelHeight);
        }

        if (typeof properties.pixelCount !== "undefined") {
            this.setPixelCount(properties.pixelCount);
        }

        if (typeof properties.pixelDensity !== "undefined") {
            this.setPixelDensity(properties.pixelDensity);
        }

        if (typeof properties.ratio !== "undefined") {
            this.setRatio(properties.ratio);
        }

        if (typeof properties.physicalWidth !== "undefined") {
            this.setPhysicalWidth(properties.physicalWidth);
        }

        if (typeof properties.physicalHeight !== "undefined") {
            this.setPhysicalHeight(properties.physicalHeight);
        }

        if (typeof properties.diagonalSize !== "undefined") {
            this.setDiagonalSize(properties.diagonalSize);
        }
    };

    /**
    * Attempts to calculate/return the pixel height of the screen.
    * Returns null if there is insufficient data.
    */
    ScreenCalc.prototype.getPixelHeight = function () {
        var ratio = this.getRatio();

        if (ratio !== null) {
            if (this.pixelWidth !== null) {
                return this.pixelWidth / ratio;
            } else if (this.pixelCount !== null) {
                return ScreenMath.pixelHeightFromRatioAndPixelCount(ratio, this.pixelCount);
            } else if (this.diagonalSize !== null && this.pixelDensity !== null) {
                var physicalHeight = ScreenMath.physicalHeightFromRatioAndDiagonalSize(ratio, this.diagonalSize);
                return physicalHeight * this.pixelDensity;
            }
        }

        if (this.physicalHeight !== null && this.pixelDensity !== null) {
            return this.physicalHeight * this.pixelDensity;
        }

        return this.pixelHeight;
    };

    /**
    * Attempts to calculate/return the pixel width of the screen.
    * Returns null if there is insufficient data.
    */
    ScreenCalc.prototype.getPixelWidth = function () {
        if (this.getPixelHeight() !== null && this.getRatio() !== null) {
            return this.getPixelHeight() * this.getRatio();
        } else if (this.physicalWidth !== null && this.pixelDensity !== null) {
            return this.physicalWidth * this.pixelDensity;
        }

        return this.pixelWidth;
    };

    /**
    * Attempts to calculate/return the screen's physical height.
    * Returns null if there is insufficient data.
    */
    ScreenCalc.prototype.getPhysicalHeight = function () {
        if (this.getRatio() !== null && this.physicalWidth !== null) {
            return this.physicalWidth / this.getRatio();
        } else if (this.pixelDensity !== null && this.pixelHeight !== null) {
            return this.pixelHeight / this.pixelDensity;
        } else if (this.getRatio() !== null && this.diagonalSize !== null) {
            return ScreenMath.physicalHeightFromRatioAndDiagonalSize(this.getRatio(), this.diagonalSize);
        }

        return this.physicalHeight;
    };

    /**
    * Attempts to calculate/return the screen's physical width.
    * Returns null if there is insufficient data.
    */
    ScreenCalc.prototype.getPhysicalWidth = function () {
        if (this.pixelDensity !== null && this.pixelWidth !== null) {
            return this.pixelWidth / this.pixelDensity;
        } else if (this.getPhysicalHeight() !== null && this.getRatio() !== null) {
            return this.getPhysicalHeight() * this.getRatio();
        }

        return this.physicalWidth;
    };

    /**
    * Attempts to calculate/return the screen's diagonal size.
    * Returns null if there is insufficient data.
    */
    ScreenCalc.prototype.getDiagonalSize = function () {
        if (this.getPhysicalHeight() !== null && this.getPhysicalWidth() !== null) {
            var diagonalSq = Math.pow(this.getPhysicalHeight(), 2) + Math.pow(this.getPhysicalWidth(), 2);
            return Math.sqrt(diagonalSq);
        }

        return this.diagonalSize;
    };

    /**
    * Returns the number of pixels per unit in the display
    * (will be ppi if units are in, or ppcm if units are cm)
    */
    ScreenCalc.prototype.getPixelDensity = function () {
        if (this.getPixelHeight() !== null && this.getPhysicalHeight() !== null) {
            return this.getPixelHeight() / this.getPhysicalHeight();
        } else if (this.getPixelWidth() !== null && this.getPhysicalWidth() !== null) {
            return this.getPixelWidth() / this.getPhysicalWidth();
        }

        return this.pixelDensity;
    };

    /** Returns the area of the display in square units */
    ScreenCalc.prototype.getArea = function () {
        return this.getPhysicalHeight() * this.getPhysicalWidth();
    };

    /** Returns the total number of pixels in the screen */
    ScreenCalc.prototype.getPixelCount = function () {
        if (this.getPixelWidth() !== null && this.getPixelHeight() !== null) {
            return this.getPixelWidth() * this.getPixelHeight();
        }

        return this.pixelCount;
    };

    /**
    * Returns the screen's width divided by its height.
    * For example, the ratio of a 1920x1080 display would
    * be 1.78 when rounded to two decimal places.
    */
    ScreenCalc.prototype.getRatio = function () {
        // try to calculate the ratio from width and height if possible
        var wAndH = this.calculateWidthAndHeight();

        if (wAndH !== null) {
            return wAndH.width / wAndH.height;
        }

        return this.ratio;
    };

    /**
    * Returns an object with 'width' and 'height' properties containing a simplified ratio for the display.
    * Additionally, a 'difference' property is included which contains the difference between the simplified
    * and original ratio (0 if the simplified ratio is exact). For example, a 1366x768 display would (by default)
    * return { width: 16, height: 9, difference: -0.0008680555555555802 }.
    * Returns null if there is not enough data to calculate the ratio.
    * @param precision (optional) pass a number between -1 and 0 for precision (defaults to 5.0e-3).
    * The closer the number is to zero the greater the precision. For example, if 1.0e-6 is passed,
    * the return value for a 1366x768 display would be { width: 683, height: 384, difference: 0 }.
    */
    ScreenCalc.prototype.getSimpleRatio = function (precision) {
        if (typeof precision === "undefined") { precision = ScreenCalc.DEFAULT_RATIO_PRECISION; }
        var ratio = this.getRatio();

        if (ratio !== null) {
            var fractionArr = ScreenMath.calculateSimplestFraction(ratio, precision);
            var simpleWidth = fractionArr[0];
            var simpleHeight = fractionArr[1];

            // 8:5 ratios should be converted to 16:10
            if (simpleWidth / simpleHeight === 16 / 10) {
                simpleWidth = 16;
                simpleHeight = 10;
            } else if (simpleWidth / simpleHeight === 10 / 16) {
                simpleWidth = 10;
                simpleHeight = 16;
            }

            var difference = (simpleWidth / simpleHeight) - ratio;
            return { width: simpleWidth, height: simpleHeight, difference: difference };
        }

        return null;
    };

    /**
    * Returns the same simplified ratio as getSimpleRatio(), but as a string in the format width:height.
    * For example, a 1920x1080 display would return "16:9". If the ratio is imprecise, a tilde (~) character
    * is prepended to the string (i.e. "~16x9").
    * Returns null if there is not enough data to calculate the ratio.
    * @param precision (optional) pass a number between -1 and 0 for precision (defaults to 5.0e-3).
    * The closer the number is to zero the greater the precision. For example, if 1.0e-6 is passed,
    * the return value for a 1366x768 display would be the precise ratio "683:384".
    */
    ScreenCalc.prototype.getStringRatio = function (precision) {
        if (typeof precision === "undefined") { precision = ScreenCalc.DEFAULT_RATIO_PRECISION; }
        var ratio = this.getSimpleRatio(precision);

        if (ratio !== null) {
            var strRatio = ratio.width.toString() + ':' + ratio.height.toString();

            if (ratio.difference !== 0) {
                strRatio = "~" + strRatio;
            }

            return strRatio;
        }

        return null;
    };

    /**
    * Tries to calculate width and height in the same units (used for ratio calculation).
    * Returns an object with height and width properties, or null if insufficient data.
    */
    ScreenCalc.prototype.calculateWidthAndHeight = function () {
        if (this.pixelWidth !== null && this.pixelHeight !== null) {
            return { width: this.pixelWidth, height: this.pixelHeight };
        } else if (this.physicalWidth !== null && this.physicalHeight !== null) {
            return { width: this.physicalWidth, height: this.physicalHeight };
        }

        if (this.pixelDensity !== null) {
            if (this.pixelWidth !== null && this.physicalHeight !== null) {
                var physicalWidth = this.pixelWidth / this.pixelDensity;
                return { width: physicalWidth, height: this.physicalHeight };
            } else if (this.pixelHeight !== null && this.physicalWidth !== null) {
                var physicalHeight = this.pixelHeight / this.pixelDensity;
                return { width: this.physicalWidth, height: physicalHeight };
            }
        }

        if (this.pixelCount !== null) {
            if (this.pixelHeight !== null) {
                var ratio = ScreenMath.ratioFromPixelHeightAndPixelCount(this.pixelHeight, this.pixelCount);
                return { height: this.pixelHeight, width: this.pixelHeight * ratio };
            } else if (this.pixelWidth !== null) {
                var pixelHeight = this.pixelCount / this.pixelWidth;
                return { height: pixelHeight, width: this.pixelWidth };
            }
        }

        if (this.diagonalSize !== null) {
            // Pythagorean theorem: width squared + height squared = diagaonl squared
            var diagonalSq = Math.pow(this.diagonalSize, 2);

            if (this.physicalWidth !== null) {
                var widthSq = Math.pow(this.physicalWidth, 2);
                var heightSq = diagonalSq - widthSq;
                return { width: this.physicalWidth, height: Math.sqrt(heightSq) };
            } else if (this.physicalHeight !== null) {
                var heightSq = Math.pow(this.physicalHeight, 2);
                var widthSq = diagonalSq - heightSq;
                return { width: Math.sqrt(widthSq), height: this.physicalHeight };
            }
        }

        return null;
    };

    ScreenCalc.prototype.setPixelWidth = function (pixelWidth) {
        if (!ScreenMath.isPositiveInt(pixelWidth)) {
            throw new Error("pixelWidth must be a positive integer");
        } else {
            this.pixelWidth = pixelWidth;
        }
    };

    ScreenCalc.prototype.setPixelHeight = function (pixelHeight) {
        if (!ScreenMath.isPositiveInt(pixelHeight)) {
            throw new Error("pixelHeight must be a positive integer");
        } else {
            this.pixelHeight = pixelHeight;
        }
    };

    ScreenCalc.prototype.setPixelCount = function (pixels) {
        if (!ScreenMath.isPositiveInt(pixels)) {
            throw new Error('pixelCount must be a positive integer');
        } else {
            this.pixelCount = pixels;
        }
    };

    ScreenCalc.prototype.setPixelDensity = function (density) {
        if (!ScreenMath.isPositiveNum(density)) {
            throw new Error('pixelDensity must be a positive number');
        } else {
            this.pixelDensity = density;
        }
    };

    ScreenCalc.prototype.setRatio = function (ratio) {
        if (!ScreenMath.isPositiveNum(ratio)) {
            throw new Error('ratio must be a positive number');
        } else {
            this.ratio = ratio;
        }
    };

    ScreenCalc.prototype.setPhysicalWidth = function (width) {
        if (!ScreenMath.isPositiveNum(width)) {
            throw new Error('physicalWidth must be a positive number');
        } else {
            this.physicalWidth = width;
        }
    };

    ScreenCalc.prototype.setPhysicalHeight = function (height) {
        if (!ScreenMath.isPositiveNum(height)) {
            throw new Error('physicalHeight must be a positive number');
        } else {
            this.physicalHeight = height;
        }
    };

    ScreenCalc.prototype.setDiagonalSize = function (diagonalSize) {
        if (!ScreenMath.isPositiveNum(diagonalSize)) {
            diagonalSize = 0;
        }

        this.diagonalSize = diagonalSize;
    };
    ScreenCalc.DEFAULT_RATIO_PRECISION = 5.0e-3;
    return ScreenCalc;
})();

module.exports = ScreenCalc;

},{"./ScreenMath":2}],2:[function(require,module,exports){
/*
* Contains functions for calculating screen properties
*/
/** Calculate the physical height of a rectangle from its ratio and diagonal size */
function physicalHeightFromRatioAndDiagonalSize(ratio, diagonalSize) {
    /*
    * Pythagorean theorem: base squared + height squared = diagonal size squared.
    * The physical width and height aren't yet known, but the ratio is. The ratio
    * can be used to define the base in terms of height, and solve for height using
    * the diagonal size.
    *
    * height squared + (ratio * height) squared = diagonal squared
    * Expanded: height squared + ratio squared * height squared = diagonal squared
    *
    * Since ratio squared is multiplied by height squared, adding height squared
    * simply increases this multiple by 1. Therefore:
    * (ratio squared + 1) * height squared = diagonal squared
    */
    var ratioSquared = Math.pow(ratio, 2);
    var diagonalSquared = Math.pow(diagonalSize, 2);
    var baseSquared = ratioSquared + 1;
    var heightSquared = diagonalSquared / baseSquared;
    return Math.sqrt(heightSquared);
}
exports.physicalHeightFromRatioAndDiagonalSize = physicalHeightFromRatioAndDiagonalSize;

/** Get the pixel height of a screen given its ratio and total number of pixels */
function pixelHeightFromRatioAndPixelCount(ratio, pixelCount) {
    /*
    * Formula:  pixelHeight * ratio * pixelHeight = pixelCount
    * Simplify: ratio * (pixelHeight)^2 = pixelCount
    * Solve:    pixelHeight = sqrt( pixelCount / ratio )
    */
    return Math.sqrt(pixelCount / ratio);
}
exports.pixelHeightFromRatioAndPixelCount = pixelHeightFromRatioAndPixelCount;

/** Get the ratio of a screen given its height in pixels and total number of pixels */
function ratioFromPixelHeightAndPixelCount(pixelHeight, pixelCount) {
    /*
    * Formula:  pixelHeight * ratio * pixelHeight = pixelCount
    * Simplify: ratio * (pixelHeight)^2 = pixelCount
    * Solve:    ratio = pixelCount / (pixelHeight)^2
    */
    return pixelCount / Math.pow(pixelHeight, 2);
}
exports.ratioFromPixelHeightAndPixelCount = ratioFromPixelHeightAndPixelCount;

/**
* Calculates the simplest fraction for a floating point number using continued fractions
* (based on http://en.wikipedia.org/wiki/Continued_fraction#Infinite_continued_fractions)
* Returns an array containing the numerator and denominator of the simplified fraction.
* @param f The number to calculate the simplest fraction for
* @param epsilon (optional) a number between -1 and 0 which determines precision (closer to zero = greater precision)
*/
function calculateSimplestFraction(f, epsilon) {
    if (typeof epsilon === "undefined") { epsilon = 5.0e-3; }
    var a = Math.floor(f);

    // convergents
    var h;
    var k;

    // the two previous convergents are necessary to
    // incorporate new terms into a rational approximation
    var h1, h2, k1, k2;

    // convergents for first 2 terms are 0/1 and 1/0
    h2 = 0;
    k2 = 1;
    h1 = 1;
    k1 = 0;

    h = a;
    k = 1;

    while (f - a > epsilon * k * k) {
        f = 1 / (f - a);
        a = Math.floor(f);

        h2 = h1;
        h1 = h;
        k2 = k1;
        k1 = k;

        // successive convergents are given by the formula:
        // h(n) = a(n)h(n-1) + h(n-2)
        // k(n)   a(n)k(n-1) + k(n-2)
        h = a * h1 + h2;
        k = a * k1 + k2;
    }

    return [h, k];
}
exports.calculateSimplestFraction = calculateSimplestFraction;

/** Returns true if the specified value is a positive integer */
function isPositiveInt(val) {
    var y = parseInt(val);
    if (isNaN(y)) {
        return false;
    }

    return val === y && val.toString() === y.toString() && val > 0;
}
exports.isPositiveInt = isPositiveInt;

/** Returns true if the value is a positive number */
function isPositiveNum(val) {
    return typeof val === "number" && val > 0;
}
exports.isPositiveNum = isPositiveNum;

},{}],3:[function(require,module,exports){
// http://wiki.commonjs.org/wiki/Unit_Testing/1.0
//
// THIS IS NOT TESTED NOR LIKELY TO WORK OUTSIDE V8!
//
// Originally from narwhal.js (http://narwhaljs.org)
// Copyright (c) 2009 Thomas Robinson <280north.com>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the 'Software'), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
// ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

// when used in node, this will actually load the util module we depend on
// versus loading the builtin util module as happens otherwise
// this is a bug in node module loading as far as I am concerned
var util = require('util/');

var pSlice = Array.prototype.slice;
var hasOwn = Object.prototype.hasOwnProperty;

// 1. The assert module provides functions that throw
// AssertionError's when particular conditions are not met. The
// assert module must conform to the following interface.

var assert = module.exports = ok;

// 2. The AssertionError is defined in assert.
// new assert.AssertionError({ message: message,
//                             actual: actual,
//                             expected: expected })

assert.AssertionError = function AssertionError(options) {
  this.name = 'AssertionError';
  this.actual = options.actual;
  this.expected = options.expected;
  this.operator = options.operator;
  if (options.message) {
    this.message = options.message;
    this.generatedMessage = false;
  } else {
    this.message = getMessage(this);
    this.generatedMessage = true;
  }
  var stackStartFunction = options.stackStartFunction || fail;

  if (Error.captureStackTrace) {
    Error.captureStackTrace(this, stackStartFunction);
  }
  else {
    // non v8 browsers so we can have a stacktrace
    var err = new Error();
    if (err.stack) {
      var out = err.stack;

      // try to strip useless frames
      var fn_name = stackStartFunction.name;
      var idx = out.indexOf('\n' + fn_name);
      if (idx >= 0) {
        // once we have located the function frame
        // we need to strip out everything before it (and its line)
        var next_line = out.indexOf('\n', idx + 1);
        out = out.substring(next_line + 1);
      }

      this.stack = out;
    }
  }
};

// assert.AssertionError instanceof Error
util.inherits(assert.AssertionError, Error);

function replacer(key, value) {
  if (util.isUndefined(value)) {
    return '' + value;
  }
  if (util.isNumber(value) && (isNaN(value) || !isFinite(value))) {
    return value.toString();
  }
  if (util.isFunction(value) || util.isRegExp(value)) {
    return value.toString();
  }
  return value;
}

function truncate(s, n) {
  if (util.isString(s)) {
    return s.length < n ? s : s.slice(0, n);
  } else {
    return s;
  }
}

function getMessage(self) {
  return truncate(JSON.stringify(self.actual, replacer), 128) + ' ' +
         self.operator + ' ' +
         truncate(JSON.stringify(self.expected, replacer), 128);
}

// At present only the three keys mentioned above are used and
// understood by the spec. Implementations or sub modules can pass
// other keys to the AssertionError's constructor - they will be
// ignored.

// 3. All of the following functions must throw an AssertionError
// when a corresponding condition is not met, with a message that
// may be undefined if not provided.  All assertion methods provide
// both the actual and expected values to the assertion error for
// display purposes.

function fail(actual, expected, message, operator, stackStartFunction) {
  throw new assert.AssertionError({
    message: message,
    actual: actual,
    expected: expected,
    operator: operator,
    stackStartFunction: stackStartFunction
  });
}

// EXTENSION! allows for well behaved errors defined elsewhere.
assert.fail = fail;

// 4. Pure assertion tests whether a value is truthy, as determined
// by !!guard.
// assert.ok(guard, message_opt);
// This statement is equivalent to assert.equal(true, !!guard,
// message_opt);. To test strictly for the value true, use
// assert.strictEqual(true, guard, message_opt);.

function ok(value, message) {
  if (!value) fail(value, true, message, '==', assert.ok);
}
assert.ok = ok;

// 5. The equality assertion tests shallow, coercive equality with
// ==.
// assert.equal(actual, expected, message_opt);

assert.equal = function equal(actual, expected, message) {
  if (actual != expected) fail(actual, expected, message, '==', assert.equal);
};

// 6. The non-equality assertion tests for whether two objects are not equal
// with != assert.notEqual(actual, expected, message_opt);

assert.notEqual = function notEqual(actual, expected, message) {
  if (actual == expected) {
    fail(actual, expected, message, '!=', assert.notEqual);
  }
};

// 7. The equivalence assertion tests a deep equality relation.
// assert.deepEqual(actual, expected, message_opt);

assert.deepEqual = function deepEqual(actual, expected, message) {
  if (!_deepEqual(actual, expected)) {
    fail(actual, expected, message, 'deepEqual', assert.deepEqual);
  }
};

function _deepEqual(actual, expected) {
  // 7.1. All identical values are equivalent, as determined by ===.
  if (actual === expected) {
    return true;

  } else if (util.isBuffer(actual) && util.isBuffer(expected)) {
    if (actual.length != expected.length) return false;

    for (var i = 0; i < actual.length; i++) {
      if (actual[i] !== expected[i]) return false;
    }

    return true;

  // 7.2. If the expected value is a Date object, the actual value is
  // equivalent if it is also a Date object that refers to the same time.
  } else if (util.isDate(actual) && util.isDate(expected)) {
    return actual.getTime() === expected.getTime();

  // 7.3 If the expected value is a RegExp object, the actual value is
  // equivalent if it is also a RegExp object with the same source and
  // properties (`global`, `multiline`, `lastIndex`, `ignoreCase`).
  } else if (util.isRegExp(actual) && util.isRegExp(expected)) {
    return actual.source === expected.source &&
           actual.global === expected.global &&
           actual.multiline === expected.multiline &&
           actual.lastIndex === expected.lastIndex &&
           actual.ignoreCase === expected.ignoreCase;

  // 7.4. Other pairs that do not both pass typeof value == 'object',
  // equivalence is determined by ==.
  } else if (!util.isObject(actual) && !util.isObject(expected)) {
    return actual == expected;

  // 7.5 For all other Object pairs, including Array objects, equivalence is
  // determined by having the same number of owned properties (as verified
  // with Object.prototype.hasOwnProperty.call), the same set of keys
  // (although not necessarily the same order), equivalent values for every
  // corresponding key, and an identical 'prototype' property. Note: this
  // accounts for both named and indexed properties on Arrays.
  } else {
    return objEquiv(actual, expected);
  }
}

function isArguments(object) {
  return Object.prototype.toString.call(object) == '[object Arguments]';
}

function objEquiv(a, b) {
  if (util.isNullOrUndefined(a) || util.isNullOrUndefined(b))
    return false;
  // an identical 'prototype' property.
  if (a.prototype !== b.prototype) return false;
  //~~~I've managed to break Object.keys through screwy arguments passing.
  //   Converting to array solves the problem.
  if (isArguments(a)) {
    if (!isArguments(b)) {
      return false;
    }
    a = pSlice.call(a);
    b = pSlice.call(b);
    return _deepEqual(a, b);
  }
  try {
    var ka = objectKeys(a),
        kb = objectKeys(b),
        key, i;
  } catch (e) {//happens when one is a string literal and the other isn't
    return false;
  }
  // having the same number of owned properties (keys incorporates
  // hasOwnProperty)
  if (ka.length != kb.length)
    return false;
  //the same set of keys (although not necessarily the same order),
  ka.sort();
  kb.sort();
  //~~~cheap key test
  for (i = ka.length - 1; i >= 0; i--) {
    if (ka[i] != kb[i])
      return false;
  }
  //equivalent values for every corresponding key, and
  //~~~possibly expensive deep test
  for (i = ka.length - 1; i >= 0; i--) {
    key = ka[i];
    if (!_deepEqual(a[key], b[key])) return false;
  }
  return true;
}

// 8. The non-equivalence assertion tests for any deep inequality.
// assert.notDeepEqual(actual, expected, message_opt);

assert.notDeepEqual = function notDeepEqual(actual, expected, message) {
  if (_deepEqual(actual, expected)) {
    fail(actual, expected, message, 'notDeepEqual', assert.notDeepEqual);
  }
};

// 9. The strict equality assertion tests strict equality, as determined by ===.
// assert.strictEqual(actual, expected, message_opt);

assert.strictEqual = function strictEqual(actual, expected, message) {
  if (actual !== expected) {
    fail(actual, expected, message, '===', assert.strictEqual);
  }
};

// 10. The strict non-equality assertion tests for strict inequality, as
// determined by !==.  assert.notStrictEqual(actual, expected, message_opt);

assert.notStrictEqual = function notStrictEqual(actual, expected, message) {
  if (actual === expected) {
    fail(actual, expected, message, '!==', assert.notStrictEqual);
  }
};

function expectedException(actual, expected) {
  if (!actual || !expected) {
    return false;
  }

  if (Object.prototype.toString.call(expected) == '[object RegExp]') {
    return expected.test(actual);
  } else if (actual instanceof expected) {
    return true;
  } else if (expected.call({}, actual) === true) {
    return true;
  }

  return false;
}

function _throws(shouldThrow, block, expected, message) {
  var actual;

  if (util.isString(expected)) {
    message = expected;
    expected = null;
  }

  try {
    block();
  } catch (e) {
    actual = e;
  }

  message = (expected && expected.name ? ' (' + expected.name + ').' : '.') +
            (message ? ' ' + message : '.');

  if (shouldThrow && !actual) {
    fail(actual, expected, 'Missing expected exception' + message);
  }

  if (!shouldThrow && expectedException(actual, expected)) {
    fail(actual, expected, 'Got unwanted exception' + message);
  }

  if ((shouldThrow && actual && expected &&
      !expectedException(actual, expected)) || (!shouldThrow && actual)) {
    throw actual;
  }
}

// 11. Expected to throw an error:
// assert.throws(block, Error_opt, message_opt);

assert.throws = function(block, /*optional*/error, /*optional*/message) {
  _throws.apply(this, [true].concat(pSlice.call(arguments)));
};

// EXTENSION! This is annoying to write outside this module.
assert.doesNotThrow = function(block, /*optional*/message) {
  _throws.apply(this, [false].concat(pSlice.call(arguments)));
};

assert.ifError = function(err) { if (err) {throw err;}};

var objectKeys = Object.keys || function (obj) {
  var keys = [];
  for (var key in obj) {
    if (hasOwn.call(obj, key)) keys.push(key);
  }
  return keys;
};

},{"util/":7}],4:[function(require,module,exports){
if (typeof Object.create === 'function') {
  // implementation from standard node.js 'util' module
  module.exports = function inherits(ctor, superCtor) {
    ctor.super_ = superCtor
    ctor.prototype = Object.create(superCtor.prototype, {
      constructor: {
        value: ctor,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
  };
} else {
  // old school shim for old browsers
  module.exports = function inherits(ctor, superCtor) {
    ctor.super_ = superCtor
    var TempCtor = function () {}
    TempCtor.prototype = superCtor.prototype
    ctor.prototype = new TempCtor()
    ctor.prototype.constructor = ctor
  }
}

},{}],5:[function(require,module,exports){
// shim for using process in browser

var process = module.exports = {};

process.nextTick = (function () {
    var canSetImmediate = typeof window !== 'undefined'
    && window.setImmediate;
    var canPost = typeof window !== 'undefined'
    && window.postMessage && window.addEventListener
    ;

    if (canSetImmediate) {
        return function (f) { return window.setImmediate(f) };
    }

    if (canPost) {
        var queue = [];
        window.addEventListener('message', function (ev) {
            if (ev.source === window && ev.data === 'process-tick') {
                ev.stopPropagation();
                if (queue.length > 0) {
                    var fn = queue.shift();
                    fn();
                }
            }
        }, true);

        return function nextTick(fn) {
            queue.push(fn);
            window.postMessage('process-tick', '*');
        };
    }

    return function nextTick(fn) {
        setTimeout(fn, 0);
    };
})();

process.title = 'browser';
process.browser = true;
process.env = {};
process.argv = [];

process.binding = function (name) {
    throw new Error('process.binding is not supported');
}

// TODO(shtylman)
process.cwd = function () { return '/' };
process.chdir = function (dir) {
    throw new Error('process.chdir is not supported');
};

},{}],6:[function(require,module,exports){
module.exports = function isBuffer(arg) {
  return arg && typeof arg === 'object'
    && typeof arg.copy === 'function'
    && typeof arg.fill === 'function'
    && typeof arg.readUInt8 === 'function';
}
},{}],7:[function(require,module,exports){
var process=require("__browserify_process"),global=typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {};// Copyright Joyent, Inc. and other Node contributors.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

var formatRegExp = /%[sdj%]/g;
exports.format = function(f) {
  if (!isString(f)) {
    var objects = [];
    for (var i = 0; i < arguments.length; i++) {
      objects.push(inspect(arguments[i]));
    }
    return objects.join(' ');
  }

  var i = 1;
  var args = arguments;
  var len = args.length;
  var str = String(f).replace(formatRegExp, function(x) {
    if (x === '%%') return '%';
    if (i >= len) return x;
    switch (x) {
      case '%s': return String(args[i++]);
      case '%d': return Number(args[i++]);
      case '%j':
        try {
          return JSON.stringify(args[i++]);
        } catch (_) {
          return '[Circular]';
        }
      default:
        return x;
    }
  });
  for (var x = args[i]; i < len; x = args[++i]) {
    if (isNull(x) || !isObject(x)) {
      str += ' ' + x;
    } else {
      str += ' ' + inspect(x);
    }
  }
  return str;
};


// Mark that a method should not be used.
// Returns a modified function which warns once by default.
// If --no-deprecation is set, then it is a no-op.
exports.deprecate = function(fn, msg) {
  // Allow for deprecating things in the process of starting up.
  if (isUndefined(global.process)) {
    return function() {
      return exports.deprecate(fn, msg).apply(this, arguments);
    };
  }

  if (process.noDeprecation === true) {
    return fn;
  }

  var warned = false;
  function deprecated() {
    if (!warned) {
      if (process.throwDeprecation) {
        throw new Error(msg);
      } else if (process.traceDeprecation) {
        console.trace(msg);
      } else {
        console.error(msg);
      }
      warned = true;
    }
    return fn.apply(this, arguments);
  }

  return deprecated;
};


var debugs = {};
var debugEnviron;
exports.debuglog = function(set) {
  if (isUndefined(debugEnviron))
    debugEnviron = process.env.NODE_DEBUG || '';
  set = set.toUpperCase();
  if (!debugs[set]) {
    if (new RegExp('\\b' + set + '\\b', 'i').test(debugEnviron)) {
      var pid = process.pid;
      debugs[set] = function() {
        var msg = exports.format.apply(exports, arguments);
        console.error('%s %d: %s', set, pid, msg);
      };
    } else {
      debugs[set] = function() {};
    }
  }
  return debugs[set];
};


/**
 * Echos the value of a value. Trys to print the value out
 * in the best way possible given the different types.
 *
 * @param {Object} obj The object to print out.
 * @param {Object} opts Optional options object that alters the output.
 */
/* legacy: obj, showHidden, depth, colors*/
function inspect(obj, opts) {
  // default options
  var ctx = {
    seen: [],
    stylize: stylizeNoColor
  };
  // legacy...
  if (arguments.length >= 3) ctx.depth = arguments[2];
  if (arguments.length >= 4) ctx.colors = arguments[3];
  if (isBoolean(opts)) {
    // legacy...
    ctx.showHidden = opts;
  } else if (opts) {
    // got an "options" object
    exports._extend(ctx, opts);
  }
  // set default options
  if (isUndefined(ctx.showHidden)) ctx.showHidden = false;
  if (isUndefined(ctx.depth)) ctx.depth = 2;
  if (isUndefined(ctx.colors)) ctx.colors = false;
  if (isUndefined(ctx.customInspect)) ctx.customInspect = true;
  if (ctx.colors) ctx.stylize = stylizeWithColor;
  return formatValue(ctx, obj, ctx.depth);
}
exports.inspect = inspect;


// http://en.wikipedia.org/wiki/ANSI_escape_code#graphics
inspect.colors = {
  'bold' : [1, 22],
  'italic' : [3, 23],
  'underline' : [4, 24],
  'inverse' : [7, 27],
  'white' : [37, 39],
  'grey' : [90, 39],
  'black' : [30, 39],
  'blue' : [34, 39],
  'cyan' : [36, 39],
  'green' : [32, 39],
  'magenta' : [35, 39],
  'red' : [31, 39],
  'yellow' : [33, 39]
};

// Don't use 'blue' not visible on cmd.exe
inspect.styles = {
  'special': 'cyan',
  'number': 'yellow',
  'boolean': 'yellow',
  'undefined': 'grey',
  'null': 'bold',
  'string': 'green',
  'date': 'magenta',
  // "name": intentionally not styling
  'regexp': 'red'
};


function stylizeWithColor(str, styleType) {
  var style = inspect.styles[styleType];

  if (style) {
    return '\u001b[' + inspect.colors[style][0] + 'm' + str +
           '\u001b[' + inspect.colors[style][1] + 'm';
  } else {
    return str;
  }
}


function stylizeNoColor(str, styleType) {
  return str;
}


function arrayToHash(array) {
  var hash = {};

  array.forEach(function(val, idx) {
    hash[val] = true;
  });

  return hash;
}


function formatValue(ctx, value, recurseTimes) {
  // Provide a hook for user-specified inspect functions.
  // Check that value is an object with an inspect function on it
  if (ctx.customInspect &&
      value &&
      isFunction(value.inspect) &&
      // Filter out the util module, it's inspect function is special
      value.inspect !== exports.inspect &&
      // Also filter out any prototype objects using the circular check.
      !(value.constructor && value.constructor.prototype === value)) {
    var ret = value.inspect(recurseTimes, ctx);
    if (!isString(ret)) {
      ret = formatValue(ctx, ret, recurseTimes);
    }
    return ret;
  }

  // Primitive types cannot have properties
  var primitive = formatPrimitive(ctx, value);
  if (primitive) {
    return primitive;
  }

  // Look up the keys of the object.
  var keys = Object.keys(value);
  var visibleKeys = arrayToHash(keys);

  if (ctx.showHidden) {
    keys = Object.getOwnPropertyNames(value);
  }

  // IE doesn't make error fields non-enumerable
  // http://msdn.microsoft.com/en-us/library/ie/dww52sbt(v=vs.94).aspx
  if (isError(value)
      && (keys.indexOf('message') >= 0 || keys.indexOf('description') >= 0)) {
    return formatError(value);
  }

  // Some type of object without properties can be shortcutted.
  if (keys.length === 0) {
    if (isFunction(value)) {
      var name = value.name ? ': ' + value.name : '';
      return ctx.stylize('[Function' + name + ']', 'special');
    }
    if (isRegExp(value)) {
      return ctx.stylize(RegExp.prototype.toString.call(value), 'regexp');
    }
    if (isDate(value)) {
      return ctx.stylize(Date.prototype.toString.call(value), 'date');
    }
    if (isError(value)) {
      return formatError(value);
    }
  }

  var base = '', array = false, braces = ['{', '}'];

  // Make Array say that they are Array
  if (isArray(value)) {
    array = true;
    braces = ['[', ']'];
  }

  // Make functions say that they are functions
  if (isFunction(value)) {
    var n = value.name ? ': ' + value.name : '';
    base = ' [Function' + n + ']';
  }

  // Make RegExps say that they are RegExps
  if (isRegExp(value)) {
    base = ' ' + RegExp.prototype.toString.call(value);
  }

  // Make dates with properties first say the date
  if (isDate(value)) {
    base = ' ' + Date.prototype.toUTCString.call(value);
  }

  // Make error with message first say the error
  if (isError(value)) {
    base = ' ' + formatError(value);
  }

  if (keys.length === 0 && (!array || value.length == 0)) {
    return braces[0] + base + braces[1];
  }

  if (recurseTimes < 0) {
    if (isRegExp(value)) {
      return ctx.stylize(RegExp.prototype.toString.call(value), 'regexp');
    } else {
      return ctx.stylize('[Object]', 'special');
    }
  }

  ctx.seen.push(value);

  var output;
  if (array) {
    output = formatArray(ctx, value, recurseTimes, visibleKeys, keys);
  } else {
    output = keys.map(function(key) {
      return formatProperty(ctx, value, recurseTimes, visibleKeys, key, array);
    });
  }

  ctx.seen.pop();

  return reduceToSingleString(output, base, braces);
}


function formatPrimitive(ctx, value) {
  if (isUndefined(value))
    return ctx.stylize('undefined', 'undefined');
  if (isString(value)) {
    var simple = '\'' + JSON.stringify(value).replace(/^"|"$/g, '')
                                             .replace(/'/g, "\\'")
                                             .replace(/\\"/g, '"') + '\'';
    return ctx.stylize(simple, 'string');
  }
  if (isNumber(value))
    return ctx.stylize('' + value, 'number');
  if (isBoolean(value))
    return ctx.stylize('' + value, 'boolean');
  // For some reason typeof null is "object", so special case here.
  if (isNull(value))
    return ctx.stylize('null', 'null');
}


function formatError(value) {
  return '[' + Error.prototype.toString.call(value) + ']';
}


function formatArray(ctx, value, recurseTimes, visibleKeys, keys) {
  var output = [];
  for (var i = 0, l = value.length; i < l; ++i) {
    if (hasOwnProperty(value, String(i))) {
      output.push(formatProperty(ctx, value, recurseTimes, visibleKeys,
          String(i), true));
    } else {
      output.push('');
    }
  }
  keys.forEach(function(key) {
    if (!key.match(/^\d+$/)) {
      output.push(formatProperty(ctx, value, recurseTimes, visibleKeys,
          key, true));
    }
  });
  return output;
}


function formatProperty(ctx, value, recurseTimes, visibleKeys, key, array) {
  var name, str, desc;
  desc = Object.getOwnPropertyDescriptor(value, key) || { value: value[key] };
  if (desc.get) {
    if (desc.set) {
      str = ctx.stylize('[Getter/Setter]', 'special');
    } else {
      str = ctx.stylize('[Getter]', 'special');
    }
  } else {
    if (desc.set) {
      str = ctx.stylize('[Setter]', 'special');
    }
  }
  if (!hasOwnProperty(visibleKeys, key)) {
    name = '[' + key + ']';
  }
  if (!str) {
    if (ctx.seen.indexOf(desc.value) < 0) {
      if (isNull(recurseTimes)) {
        str = formatValue(ctx, desc.value, null);
      } else {
        str = formatValue(ctx, desc.value, recurseTimes - 1);
      }
      if (str.indexOf('\n') > -1) {
        if (array) {
          str = str.split('\n').map(function(line) {
            return '  ' + line;
          }).join('\n').substr(2);
        } else {
          str = '\n' + str.split('\n').map(function(line) {
            return '   ' + line;
          }).join('\n');
        }
      }
    } else {
      str = ctx.stylize('[Circular]', 'special');
    }
  }
  if (isUndefined(name)) {
    if (array && key.match(/^\d+$/)) {
      return str;
    }
    name = JSON.stringify('' + key);
    if (name.match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)) {
      name = name.substr(1, name.length - 2);
      name = ctx.stylize(name, 'name');
    } else {
      name = name.replace(/'/g, "\\'")
                 .replace(/\\"/g, '"')
                 .replace(/(^"|"$)/g, "'");
      name = ctx.stylize(name, 'string');
    }
  }

  return name + ': ' + str;
}


function reduceToSingleString(output, base, braces) {
  var numLinesEst = 0;
  var length = output.reduce(function(prev, cur) {
    numLinesEst++;
    if (cur.indexOf('\n') >= 0) numLinesEst++;
    return prev + cur.replace(/\u001b\[\d\d?m/g, '').length + 1;
  }, 0);

  if (length > 60) {
    return braces[0] +
           (base === '' ? '' : base + '\n ') +
           ' ' +
           output.join(',\n  ') +
           ' ' +
           braces[1];
  }

  return braces[0] + base + ' ' + output.join(', ') + ' ' + braces[1];
}


// NOTE: These type checking functions intentionally don't use `instanceof`
// because it is fragile and can be easily faked with `Object.create()`.
function isArray(ar) {
  return Array.isArray(ar);
}
exports.isArray = isArray;

function isBoolean(arg) {
  return typeof arg === 'boolean';
}
exports.isBoolean = isBoolean;

function isNull(arg) {
  return arg === null;
}
exports.isNull = isNull;

function isNullOrUndefined(arg) {
  return arg == null;
}
exports.isNullOrUndefined = isNullOrUndefined;

function isNumber(arg) {
  return typeof arg === 'number';
}
exports.isNumber = isNumber;

function isString(arg) {
  return typeof arg === 'string';
}
exports.isString = isString;

function isSymbol(arg) {
  return typeof arg === 'symbol';
}
exports.isSymbol = isSymbol;

function isUndefined(arg) {
  return arg === void 0;
}
exports.isUndefined = isUndefined;

function isRegExp(re) {
  return isObject(re) && objectToString(re) === '[object RegExp]';
}
exports.isRegExp = isRegExp;

function isObject(arg) {
  return typeof arg === 'object' && arg !== null;
}
exports.isObject = isObject;

function isDate(d) {
  return isObject(d) && objectToString(d) === '[object Date]';
}
exports.isDate = isDate;

function isError(e) {
  return isObject(e) &&
      (objectToString(e) === '[object Error]' || e instanceof Error);
}
exports.isError = isError;

function isFunction(arg) {
  return typeof arg === 'function';
}
exports.isFunction = isFunction;

function isPrimitive(arg) {
  return arg === null ||
         typeof arg === 'boolean' ||
         typeof arg === 'number' ||
         typeof arg === 'string' ||
         typeof arg === 'symbol' ||  // ES6 symbol
         typeof arg === 'undefined';
}
exports.isPrimitive = isPrimitive;

exports.isBuffer = require('./support/isBuffer');

function objectToString(o) {
  return Object.prototype.toString.call(o);
}


function pad(n) {
  return n < 10 ? '0' + n.toString(10) : n.toString(10);
}


var months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep',
              'Oct', 'Nov', 'Dec'];

// 26 Feb 16:19:34
function timestamp() {
  var d = new Date();
  var time = [pad(d.getHours()),
              pad(d.getMinutes()),
              pad(d.getSeconds())].join(':');
  return [d.getDate(), months[d.getMonth()], time].join(' ');
}


// log is just a thin wrapper to console.log that prepends a timestamp
exports.log = function() {
  console.log('%s - %s', timestamp(), exports.format.apply(exports, arguments));
};


/**
 * Inherit the prototype methods from one constructor into another.
 *
 * The Function.prototype.inherits from lang.js rewritten as a standalone
 * function (not on Function.prototype). NOTE: If this file is to be loaded
 * during bootstrapping this function needs to be rewritten using some native
 * functions as prototype setup using normal JavaScript does not work as
 * expected during bootstrapping (see mirror.js in r114903).
 *
 * @param {function} ctor Constructor function which needs to inherit the
 *     prototype.
 * @param {function} superCtor Constructor function to inherit prototype from.
 */
exports.inherits = require('inherits');

exports._extend = function(origin, add) {
  // Don't do anything if add isn't an object
  if (!add || !isObject(add)) return origin;

  var keys = Object.keys(add);
  var i = keys.length;
  while (i--) {
    origin[keys[i]] = add[keys[i]];
  }
  return origin;
};

function hasOwnProperty(obj, prop) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

},{"./support/isBuffer":6,"__browserify_process":5,"inherits":4}],8:[function(require,module,exports){
/// <reference path="../ts_definitions/node.d.ts" />
/// <reference path="../ts_definitions/mocha.d.ts" />
var ScreenCalc = require('../lib/ScreenCalc');
var assert = require('assert');

describe('constructor', function () {
    it('should allow displays to be constructed without any properties', function () {
        assert.doesNotThrow(function () {
            var screen = new ScreenCalc();
        });
    });

    it('should allow screens to be created with multiple properties', function () {
        assert.doesNotThrow(function () {
            var screen = new ScreenCalc({
                pixelWidth: 1920,
                pixelHeight: 1080,
                diagonalSize: 10.6
            });
        });
    });
});

describe('setData()', function () {
    it('should reset unpassed properties if "reset" is set to true', function () {
        var screen = new ScreenCalc({
            pixelCount: 1000
        });

        screen.setData({
            pixelWidth: 1024
        }, true);

        assert.strictEqual(screen.getPixelCount(), null);
        assert.strictEqual(screen.getPixelWidth(), 1024);
    });

    it('should update dependent properties if they are already set');
    it('should throw an error if conflicting data is provided');
});

describe('getPixelWidth()', function () {
    it('should work with pixel count and ratio', function () {
        var screen = new ScreenCalc({
            ratio: 1920 / 1080,
            pixelCount: 1920 * 1080
        });

        assert.strictEqual(screen.getPixelWidth(), 1920);
    });

    it('should work with physical width and density', function () {
        var ipadAir = new ScreenCalc({
            physicalWidth: 7.76,
            pixelDensity: 264
        });

        assert.strictEqual(Math.floor(ipadAir.getPixelWidth()), 2048); // 2048.64
    });
});

describe('getPixelHeight()', function () {
    it('should work with pixel count and ratio', function () {
        var screen = new ScreenCalc({
            ratio: 1920 / 1080,
            pixelCount: 1920 * 1080
        });

        assert.strictEqual(screen.getPixelHeight(), 1080);
    });

    it('should work with physical height and density', function () {
        var ipadAir = new ScreenCalc({
            physicalHeight: 5.82,
            pixelDensity: 264
        });

        assert.strictEqual(Math.round(ipadAir.getPixelHeight()), 1536);
    });

    it('should work with ratio, diagonal size, and density', function () {
        var ipadAir = new ScreenCalc({
            pixelDensity: 264,
            ratio: 4 / 3,
            diagonalSize: 9.7
        });

        assert.strictEqual(Math.round(ipadAir.getPixelHeight()), 1536);
    });

    it('should work with physical width, density, and diagonal size', function () {
        var netbook = new ScreenCalc({
            // 1024x600
            physicalWidth: 7.678914306769556,
            pixelDensity: 133.3521848391074,
            diagonalSize: 8.9
        });

        assert.strictEqual(netbook.getPixelHeight(), 600);
    });
});

describe('getPhysicalWidth()', function () {
    it('should work with resolution and diagonal size', function () {
        var ipadAir = new ScreenCalc({
            pixelWidth: 2048,
            pixelHeight: 1536,
            diagonalSize: 9.7
        });

        assert.strictEqual(ipadAir.getPhysicalWidth(), 7.76);
    });

    it('should work with diagonal size and ratio', function () {
        var asusVivotab = new ScreenCalc({
            ratio: 16 / 9,
            diagonalSize: 10.1
        });

        assert.strictEqual(Math.round(asusVivotab.getPhysicalWidth() * 100) / 100, 8.8);
    });
});

describe('getPhysicalHeight()', function () {
    it('should work when the resolution and diagonal size are known', function () {
        var ipadAir = new ScreenCalc({
            pixelWidth: 2048,
            pixelHeight: 1536,
            diagonalSize: 9.7
        });

        assert.strictEqual(ipadAir.getPhysicalHeight(), 5.82);
    });

    it('should work when the ratio and physical width are known', function () {
        var asusVivotab = new ScreenCalc({
            ratio: 16 / 9,
            physicalWidth: 8.8
        });

        assert.strictEqual(Math.round(asusVivotab.getPhysicalHeight() * 100) / 100, 4.95);
    });
});

describe('getRatio()', function () {
    it('should work with ratio', function () {
        var screen = new ScreenCalc({ ratio: 16 / 10 });
        assert.strictEqual(screen.getRatio(), 16 / 10);
    });

    it('should work with physical width and physical height', function () {
        var screen = new ScreenCalc({ physicalWidth: 16, physicalHeight: 10 });
        assert.strictEqual(screen.getRatio(), 16 / 10);
    });

    it('should work with pixel width and pixel height', function () {
        var screen = new ScreenCalc({ pixelWidth: 1600, pixelHeight: 1200 });
        assert.strictEqual(screen.getRatio(), 4 / 3);
    });

    it('should work with pixel width, physical height, and pixel density', function () {
        var ipadAir = new ScreenCalc({
            pixelWidth: 2048,
            physicalHeight: 5.82,
            pixelDensity: 1536 / 5.82
        });

        assert.strictEqual(Math.round(ipadAir.getRatio() * 100) / 100, Math.round(4 / 3 * 100) / 100);
    });

    it('should work with pixel height and total number of pixels', function () {
        var screen = new ScreenCalc({
            pixelHeight: 1200,
            pixelCount: 1600 * 1200
        });

        assert.strictEqual(screen.getRatio(), 4 / 3);
    });

    it('should work with diagonal size and physical width or height', function () {
        var screen = new ScreenCalc({
            diagonalSize: 20,
            physicalWidth: 16
        });

        var portraitScreen = new ScreenCalc({
            diagonalSize: 30,
            physicalHeight: 24
        });

        assert.strictEqual(screen.getRatio(), 4 / 3);
        assert.strictEqual(portraitScreen.getRatio(), 3 / 4);
    });
});

describe('getSimpleRatio()', function () {
    it('should work whenever ratio is available', function () {
        var display = new ScreenCalc({
            ratio: 5 / 9
        });
        assert.deepEqual(display.getSimpleRatio(), { width: 5, height: 9, difference: 0 });

        var monitor = new ScreenCalc({ pixelWidth: 1920, pixelHeight: 1080 });
        assert.deepEqual(monitor.getSimpleRatio(), { width: 16, height: 9, difference: 0 });
    });

    it('should return null if ratio is not available', function () {
        var screen = new ScreenCalc({ pixelWidth: 1024, physicalHeight: 15 });
        assert.strictEqual(screen.getSimpleRatio(), null);
    });

    it('should convert 8:5 ratios to 16:10', function () {
        var screen = new ScreenCalc({
            ratio: 16 / 10
        });

        var portraitScreen = new ScreenCalc({
            ratio: 10 / 16
        });

        assert.deepEqual(screen.getSimpleRatio(), { width: 16, height: 10, difference: 0 });
        assert.deepEqual(portraitScreen.getSimpleRatio(), { width: 10, height: 16, difference: 0 });
    });

    it('should allow precision to be changed', function () {
        var screen = new ScreenCalc({ pixelWidth: 1136, pixelHeight: 640 });
        assert.deepEqual(screen.getSimpleRatio(1.0e-3), { width: 71, height: 40, difference: 0 });
    });

    it('should include non-zero difference for imprecise ratios', function () {
        var screen = new ScreenCalc({ pixelWidth: 1136, pixelHeight: 640 });
        assert.strictEqual(screen.getSimpleRatio().difference, 0.002777777777777768);
    });
});

describe('getStringRatio()', function () {
    it('should return ratio in width:height format', function () {
        var screen = new ScreenCalc({ physicalWidth: 20, physicalHeight: 10 });
        assert.strictEqual(screen.getStringRatio(), "2:1");
    });

    it('should include a tilde if ratio is not exact', function () {
        var impreciseScreen = new ScreenCalc({ pixelWidth: 1366, pixelHeight: 768 });
        assert.strictEqual(impreciseScreen.getStringRatio(), "~16:9");
    });

    it('should allow precision to be changed', function () {
        var screen = new ScreenCalc({ ratio: 1366 / 768 });
        assert.strictEqual(screen.getStringRatio(1.0e-5), "683:384");
    });

    it('should return null if insufficient data', function () {
        var screen = new ScreenCalc();
        assert.strictEqual(screen.getStringRatio(), null);
    });
});

describe('getArea()', function () {
    // expected iPad and Asus VivoTab numbers taken from http://www.curi.us/1571-lying-microsoft-advertising
    it('should work with resolution and diagonal size', function () {
        var ipadAir = new ScreenCalc({
            pixelWidth: 2048,
            pixelHeight: 1536,
            diagonalSize: 9.7
        });

        assert.strictEqual(Math.round(ipadAir.getArea() * 100) / 100, 45.16);
    });

    it('should work with ratio, pixel width, and diagonal size', function () {
        var asusVivotab = new ScreenCalc({
            pixelWidth: 1366,
            ratio: 16 / 9,
            diagonalSize: 10.1
        });

        assert.strictEqual(Math.round(asusVivotab.getArea() * 10) / 10, 43.6);
    });

    it('should work with physical width, pixel height, and density', function () {
        var surfacePro = new ScreenCalc({
            pixelHeight: 1080,
            pixelDensity: 208,
            physicalWidth: 9.2387
        });

        assert.strictEqual(Math.round(surfacePro.getArea()), 48);
    });
});

describe('getPixelDensity()', function () {
    it('should work with pixel width and physical width', function () {
        var lumia920 = new ScreenCalc({
            pixelWidth: 1280,
            physicalWidth: 3.8587
        });

        assert.strictEqual(Math.round(lumia920.getPixelDensity() * 10) / 10, 331.7);
    });

    it('should work with pixel height and physical height', function () {
        var lumia920 = new ScreenCalc({
            pixelHeight: 768,
            physicalHeight: 2.3152
        });

        assert.strictEqual(Math.round(lumia920.getPixelDensity() * 10) / 10, 331.7);
    });

    it('should work with resolution and diagonal size', function () {
        var iphone5 = new ScreenCalc({
            pixelWidth: 1136,
            pixelHeight: 640,
            diagonalSize: 4.0
        });

        assert.strictEqual(Math.round(iphone5.getPixelDensity()), 326);
    });

    it('should work with pixel width, physical height, and ratio', function () {
        var nexus7 = new ScreenCalc({
            pixelWidth: 1920,
            physicalHeight: 3.71,
            ratio: 1920 / 1200
        });

        assert.strictEqual(Math.round(nexus7.getPixelDensity() * 100) / 100, 323.45);
    });
});

describe('getPixelCount()', function () {
    it('should work with pixel width and pixel height', function () {
        var screen = new ScreenCalc({
            pixelWidth: 640,
            pixelHeight: 480
        });

        assert.strictEqual(screen.getPixelCount(), 640 * 480);
    });

    it('should work with pixel width and ratio', function () {
        var screen = new ScreenCalc({
            pixelWidth: 640,
            ratio: 4 / 3
        });

        assert.strictEqual(screen.getPixelCount(), 640 * 480);
    });

    it('should work with pixel count', function () {
        var screen = new ScreenCalc({ pixelCount: 1000000 });
        assert.strictEqual(screen.getPixelCount(), 1000000);
    });

    it('should return null if insufficient data', function () {
        var screen = new ScreenCalc({ pixelWidth: 1024 });
        assert.strictEqual(screen.getPixelCount(), null);
    });
});

describe('getDiagonalSize()', function () {
    var ipadMiniRetina = new ScreenCalc({ pixelWidth: 2048, pixelHeight: 1536 });

    it("should return null if diagonal size and density aren't specified", function () {
        assert.strictEqual(ipadMiniRetina.getDiagonalSize(), null);
    });

    it('should correctly calculate the diagonal size when pixel density is set', function () {
        ipadMiniRetina.setData({ pixelDensity: 326 });
        assert.strictEqual(Math.round(ipadMiniRetina.getDiagonalSize() * 10) / 10, 7.9);
    });

    it('should change diagonal size to zero if set to NaN', function () {
        var impossibleDisplay = new ScreenCalc({ pixelWidth: 1000, pixelHeight: 1000, diagonalSize: NaN });
        assert.strictEqual(impossibleDisplay.getDiagonalSize(), 0);
    });
});

},{"../lib/ScreenCalc":1,"assert":3}],9:[function(require,module,exports){
/// <reference path="../ts_definitions/node.d.ts" />
/// <reference path="../ts_definitions/mocha.d.ts" />
var ScreenMath = require('../lib/ScreenMath');
var assert = require('assert');

describe('isPositiveInt()', function () {
    it('should return false for 0', function () {
        assert.strictEqual(ScreenMath.isPositiveInt(0), false);
    });

    it('should return false for negative numbers', function () {
        assert.strictEqual(ScreenMath.isPositiveInt(-0), false);
        assert.strictEqual(ScreenMath.isPositiveInt(-1), false);
        assert.strictEqual(ScreenMath.isPositiveInt(-25), false);
    });

    it('should return false for strings', function () {
        assert.strictEqual(ScreenMath.isPositiveInt("string"), false);
        assert.strictEqual(ScreenMath.isPositiveInt("1"), false);
    });

    it('should return false for floats', function () {
        assert.strictEqual(ScreenMath.isPositiveInt(1.5), false);
    });

    it('should return true for positive integers', function () {
        assert.strictEqual(ScreenMath.isPositiveInt(1), true);
        assert.strictEqual(ScreenMath.isPositiveInt(2.00), true);
        assert.strictEqual(ScreenMath.isPositiveInt(77), true);
        assert.strictEqual(ScreenMath.isPositiveInt(100000000000), true);
    });
});

describe('isPositiveNum()', function () {
    it('should return false for 0', function () {
        assert.strictEqual(ScreenMath.isPositiveNum(0), false);
    });

    it('should return false for negative numbers', function () {
        assert.strictEqual(ScreenMath.isPositiveNum(-0), false);
        assert.strictEqual(ScreenMath.isPositiveNum(-1), false);
        assert.strictEqual(ScreenMath.isPositiveNum(-25), false);
    });

    it('should return false for strings', function () {
        assert.strictEqual(ScreenMath.isPositiveNum("string"), false);
        assert.strictEqual(ScreenMath.isPositiveNum("1"), false);
        assert.strictEqual(ScreenMath.isPositiveNum("2.0"), false);
    });

    it('should return true for positive floats', function () {
        assert.strictEqual(ScreenMath.isPositiveNum(1.5), true);
        assert.strictEqual(ScreenMath.isPositiveNum(0.1593792), true);
    });

    it('should return true for positive integers', function () {
        assert.strictEqual(ScreenMath.isPositiveNum(1), true);
        assert.strictEqual(ScreenMath.isPositiveNum(77), true);
        assert.strictEqual(ScreenMath.isPositiveNum(100000000000), true);
    });
});

describe('Calculate physical height', function () {
    it('should work with ratio and diagonal size', function () {
        var ipad = {
            ratio: 2048 / 1536,
            diagonalSize: 9.7
        };

        var ipadHeight = ScreenMath.physicalHeightFromRatioAndDiagonalSize(ipad.ratio, ipad.diagonalSize);
        assert.strictEqual(ipadHeight, 5.82);

        var vivotab = {
            ratio: 1366 / 768,
            diagonalSize: 10.1
        };

        var vivotabHeight = ScreenMath.physicalHeightFromRatioAndDiagonalSize(vivotab.ratio, vivotab.diagonalSize);
        assert.strictEqual(Math.round(vivotabHeight * 100) / 100, 4.95);
    });
});

describe('Calculate pixel height', function () {
    it('should work with ratio and pixel count', function () {
        var ratio = 16 / 9;
        var pixelCount = 1920 * 1080;

        var ratio2 = 3 / 4;
        var pixelCount2 = 1536 * 2048;

        assert.strictEqual(ScreenMath.pixelHeightFromRatioAndPixelCount(ratio, pixelCount), 1080);
        assert.strictEqual(ScreenMath.pixelHeightFromRatioAndPixelCount(ratio2, pixelCount2), 2048);
    });
});

describe('Calculate ratio', function () {
    it('should work with pixel height and pixel count', function () {
        assert.strictEqual(ScreenMath.ratioFromPixelHeightAndPixelCount(1536, 2048 * 1536), 4 / 3);
    });
});

describe('Calculate simplest fraction', function () {
    it('should work with common screens with exact ratios', function () {
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1920 / 1080), [16, 9]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1920 / 1200), [8, 5]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(2048 / 1536), [4, 3]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1280 / 768), [5, 3]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(2880 / 900), [16, 5]);
    });

    it('should work with reciprocals of common screen ratios', function () {
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1080 / 1920), [9, 16]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1200 / 1920), [5, 8]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1536 / 2048), [3, 4]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(768 / 1280), [3, 5]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(900 / 2880), [5, 16]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(768 / 1366), [9, 16]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(640 / 1136), [9, 16]);
    });

    it('should round to simple ratios by default', function () {
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1366 / 768), [16, 9]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1136 / 640), [16, 9]);
    });

    it('should allow default precision to be overridden', function () {
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1366 / 768, 1.0e-5), [683, 384]);
        assert.deepEqual(ScreenMath.calculateSimplestFraction(1136 / 640, 1.0e-3), [71, 40]);
    });
});

},{"../lib/ScreenMath":2,"assert":3}]},{},[8,9])